@@ -1,6 +1,6 @@
 #include "renpy.h"
-#include <pygame/pygame.h>
 #include "IMG_savepng.h"
+#include <pygame/pygame.h>
 #include <stdio.h>
 #include <math.h>
 
@@ -30,7 +30,6 @@ void save_png_core(PyObject *pysurf, SDL_RWops *rw, int compress) {
     IMG_SavePNG_RW(rw, surf, compress);
 }
 
-
 /* This pixellates a 32-bit RGBA pygame surface to a destination
  * surface of a given size.
  *
@@ -960,13 +959,16 @@ void scale24_core(PyObject *pysrc, PyObject *pydst,
     Py_END_ALLOW_THREADS
 }
 
+#define I(a, b, mul) ((((((b - a) * mul)) >> 8) + a) & 0xff00ff)
+                
 /****************************************************************************/
 /* A similar concept to rotozoom, but implemented differently, so we
    can limit the target area. */
-void transform32_core(PyObject *pysrc, PyObject *pydst,
-                      float corner_x, float corner_y,
-                      float xdx, float ydx,
-                      float xdy, float ydy) {
+void transform32_std(PyObject *pysrc, PyObject *pydst,
+                     float corner_x, float corner_y,
+                     float xdx, float ydx,
+                     float xdy, float ydy,
+                     int ashift, float a) {
 
     SDL_Surface *src;
     SDL_Surface *dst;
@@ -986,7 +988,7 @@ void transform32_core(PyObject *pysrc, PyObject *pydst,
     dst = PySurface_AsSurface(pydst);
         
     Py_BEGIN_ALLOW_THREADS
-
+        
     srcpixels = (unsigned char *) src->pixels;
     dstpixels = (unsigned char *) dst->pixels;
     srcpitch = src->pitch;
@@ -996,20 +998,21 @@ void transform32_core(PyObject *pysrc, PyObject *pydst,
     srch = src->h;
     dsth = dst->h;
 
-    lsx = corner_x * 256;
-    lsy = corner_y * 256;
+    unsigned int amul = (int) (a * 256);
+    
+    lsx = corner_x * 65536;
+    lsy = corner_y * 65536;
 
-    xdx *= 256;
-    ydx *= 256;
-    xdy *= 256;
-    ydy *= 256;
+    xdx *= 65536;
+    ydx *= 65536;
+    xdy *= 65536;
+    ydy *= 65536;
 
 
     // Scaled subtracted srcw and srch.
-    float fsw = (srcw - 2) * 256;
-    float fsh = (srch - 2) * 256;
-    
-    
+    float fsw = (srcw - 1) * 65536 - 1;
+    float fsh = (srch - 1) * 65536 - 1;
+
     for (y = 0; y < dsth; y++, lsx += xdy, lsy += ydy) {
 
         sx = lsx;
@@ -1028,7 +1031,6 @@ void transform32_core(PyObject *pysrc, PyObject *pydst,
             minx = fmaxf(minx, fminf(d1, d2)); 
             maxx = fminf(maxx, fmaxf(d1, d2)); 
             
-//            printf("ZZZ1 %f %f\n", d1, d2);
         } else if ( lsx < 0 || lsx >= fsw) {
             continue;
         }
@@ -1039,7 +1041,6 @@ void transform32_core(PyObject *pysrc, PyObject *pydst,
 
             minx = fmaxf(minx, fminf(d1, d2)); 
             maxx = fminf(maxx, fmaxf(d1, d2)); 
-//            printf("ZZZ2 %f %f\n", d1, d2);
         } else if ( lsy < 0 || lsy >= fsh) {
             continue;
         }
@@ -1048,108 +1049,316 @@ void transform32_core(PyObject *pysrc, PyObject *pydst,
             continue;
         }
 
-//        printf("CCC %f %f\n", minx, maxx); 
-        
         minx = ceil(minx);
         maxx = floor(maxx);
 
-//        printf("CCC2 %f %f\n", minx, maxx); 
-//        printf("XXX %f %f %f %f\n", lsx / 256, xdx / 256, lsy / 256, ydx/256);
-
-        
-        // printf("%f %f\n", minx, maxx);
-        
-        // printf("Minx at: %f %f\n", (lsx + xdx * minx) / 256, (lsy + ydx * minx) / 256);
-
-        
         unsigned char *d = dstpixels + dstpitch * y;
         unsigned char *dend = d + 4 * (int) maxx;
         d += 4 * (int) minx;
 
-        sx += minx * xdx;
-        sy += minx * ydx;
-                
-        
+        sx = lsx + minx * xdx;
+        sy = lsy + minx * ydx;
+
+        int sxi = (int) sx;
+        int syi = (int) sy;
+        int xdxi = (int) xdx;
+        int ydxi = (int) ydx;
+
         while (d <= dend) {
+            int px, py;
+            px = sxi >> 16;
+            py = syi >> 16;
+                
+            unsigned char *sp = srcpixels + py * srcpitch + px * 4;
 
-            int px, py, sxi, syi;
-            sxi = ((int) sx);
-            syi = ((int) sy);                
-            px = sxi >> 8;
-            py = syi >> 8;
+            int yfrac = (syi >> 8) & 0xff; // ((short) sy) & 0xff;
+            int xfrac = (sxi >> 8) & 0xff; // ((short) sx) & 0xff;
 
+            unsigned int pal = *(unsigned int *) sp;
+            unsigned int pbl = *(unsigned int *) (sp + 4);
+            sp += srcpitch;
+            unsigned int pcl = *(unsigned int *) sp;
+            unsigned int pdl = *(unsigned int *) (sp + 4);
 
-            /* These bounds are checked analytically, hence the if (1)
-             * { */
+            unsigned int pah = (pal >> 8) & 0xff00ff;
+            unsigned int pbh = (pbl >> 8) & 0xff00ff;
+            unsigned int pch = (pcl >> 8) & 0xff00ff;
+            unsigned int pdh = (pdl >> 8) & 0xff00ff;
+
+            pal &= 0xff00ff;
+            pbl &= 0xff00ff;
+            pcl &= 0xff00ff;
+            pdl &= 0xff00ff;
+
+            unsigned int rh = I(I(pah, pch, yfrac), I(pbh, pdh, yfrac), xfrac);
+            unsigned int rl = I(I(pal, pcl, yfrac), I(pbl, pdl, yfrac), xfrac);
+
+            unsigned int alpha = (((rh << 8) | rl) >> ashift) & 0xff;
+            alpha = (alpha * amul) >> 8;
             
-            // if (0 <= px && px < srcw - 1 && 0 <= py && py < srch - 1) {
-            if (1) {
-                
-                unsigned char *sp = srcpixels + py * srcpitch + px * 4;
+            unsigned int dl = * (unsigned int *) d;
+            unsigned int dh = (dl >> 8) & 0xff00ff;
+            dl &= 0xff00ff;
 
-                // unsigned char *s1p = s0p + srcpitch;
-                
-                int s1frac = syi & 0xff; // ((short) sy) & 0xff;
-                int xfrac = sxi & 0xff; // ((short) sx) & 0xff;
-
-#define I(a, b, mul) ((((((b - a) * mul)) >> 8) + a) & 0xff00ff)
-                
-                unsigned int rl, rh;
-
-                unsigned int pal = *(unsigned int *) sp;
-                unsigned int pbl = *(unsigned int *) (sp + 4);
-                sp += srcpitch;
-                unsigned int pcl = *(unsigned int *) sp;
-                unsigned int pdl = *(unsigned int *) (sp + 4);
-
-                unsigned int pah = (pal >> 8) & 0xff00ff;
-                unsigned int pbh = (pbl >> 8) & 0xff00ff;
-                unsigned int pch = (pcl >> 8) & 0xff00ff;
-                unsigned int pdh = (pdl >> 8) & 0xff00ff;
-
-                pal &= 0xff00ff;
-                pbl &= 0xff00ff;
-                pcl &= 0xff00ff;
-                pdl &= 0xff00ff;
-
-                // if (y == 10)
-                // printf("%x %x %x %x\n", pah, pbh, pch, pdh);
-                
-
-                rh = I(I(pah, pch, s1frac), I(pbh, pdh, s1frac), xfrac);
-                rl = I(I(pal, pcl, s1frac), I(pbl, pdl, s1frac), xfrac);
-                // rl = 0;
-
-                // if (y == 10) 
-                // printf("%x\n", rh);
-                
-                * (unsigned int *) d = (rh << 8) | rl;
-
-                d += 4;
-                
-
-            } else {
-                // This code can't execute.
-
-                printf("Failed x %f (%d, %d)\n", minx, px, py);
- 
-                *d++ = 0;
-                *d++ = 0;
-                *d++ = 0;
-                *d++ = 0;                
-
-            }
+            dl = I(dl, rl, alpha);
+            dh = I(dh, rh, alpha);
             
-            sx += xdx;
-            sy += ydx;
-            // minx++;
+            * (unsigned int *) d = (dh << 8) | dl;
+
+            d += 4;
+            sxi += xdxi;
+            syi += ydxi;
         }
     }
 
     Py_END_ALLOW_THREADS
 }
 
+#ifdef GCC_MMX
 
+/****************************************************************************/
+/* A similar concept to rotozoom, but implemented differently, so we
+   can limit the target area. */
+int transform32_mmx(PyObject *pysrc, PyObject *pydst,
+                     float corner_x, float corner_y,
+                     float xdx, float ydx,
+                     float xdy, float ydy,
+                     int ashift,
+                     float a) {
+
+    SDL_Surface *src;
+    SDL_Surface *dst;
+    
+    int y;
+    int srcpitch, dstpitch;
+                               
+    int srcw, srch;
+    int dstw, dsth;
+    
+    unsigned char *srcpixels;
+    unsigned char *dstpixels;
+
+    float lsx, lsy; // The position of the current line in the source.
+    float sx, sy; // The position of the current pixel in the source.
+    unsigned int px, py;
+    unsigned int sxi, syi;
+    unsigned int xdxi, ydxi;
+        
+    src = PySurface_AsSurface(pysrc);
+    dst = PySurface_AsSurface(pydst);
+        
+    Py_BEGIN_ALLOW_THREADS
+        
+    srcpixels = (unsigned char *) src->pixels;
+    dstpixels = (unsigned char *) dst->pixels;
+    srcpitch = src->pitch;
+    dstpitch = dst->pitch;
+    srcw = src->w;
+    dstw = dst->w;
+    srch = src->h;
+    dsth = dst->h;
+
+    // Due to mmx.
+    ashift *= 2;
+
+    // Compute the coloring multiplier.
+    unsigned int amul = (unsigned int) (a * 256);
+    
+    lsx = corner_x * 65536;
+    lsy = corner_y * 65536;
+
+    xdx *= 65536;
+    ydx *= 65536;
+    xdy *= 65536;
+    ydy *= 65536;
+
+    // Scaled subtracted srcw and srch.
+    float fsw = (srcw - 1) * 65536 - 1;
+    float fsh = (srch - 1) * 65536 - 1;
+
+    for (y = 0; y < dsth; y++, lsx += xdy, lsy += ydy) {
+
+        float minx = 0;
+        float maxx = dstw - 1;
+
+        if (xdx != 0) {
+            float d1 = -lsx / xdx;
+            float d2 = (fsw - lsx) / xdx;
+
+            minx = fmaxf(minx, fminf(d1, d2)); 
+            maxx = fminf(maxx, fmaxf(d1, d2)); 
+            
+        } else if ( lsx < 0 || lsx >= fsw) {
+            continue;
+        }
+
+        if (ydx != 0) {
+            float d1 = -lsy / ydx;
+            float d2 = (fsh - lsy) / ydx;
+
+            minx = fmaxf(minx, fminf(d1, d2)); 
+            maxx = fminf(maxx, fmaxf(d1, d2)); 
+        } else if ( lsy < 0 || lsy >= fsh) {
+            continue;
+        }
+
+        if (minx > maxx) {
+            continue;
+        }
+        
+        minx = ceil(minx);
+        maxx = floor(maxx);
+        
+        unsigned char *d = dstpixels + dstpitch * y;
+        unsigned char *dend = d + 4 * (int) maxx;
+        d += 4 * (int) minx;
+
+        sx = (lsx + minx * xdx);
+        sy = (lsy + minx * ydx);
+            
+        sxi = (unsigned int) sx;
+        syi = (unsigned int) sy;
+        xdxi = (int) xdx;
+        ydxi = (int) ydx;
+
+        // No floating point allowed between here and the end of the
+        // while loop.
+
+        // 0 -> mm7
+        pxor_r2r(mm7, mm7);
+
+        // ashift -> mm0
+        movd_m2r(ashift, mm0);
+
+        // amul -> mm4
+        movd_m2r(amul, mm4);
+        
+        while (d <= dend) {
+
+            px = sxi >> 16;
+            py = syi >> 16;
+
+            unsigned char *sp = srcpixels + py * srcpitch + px * 4;
+
+            unsigned int yfrac = (syi >> 8) & 0xff; // ((short) sy) & 0xff;
+            unsigned int xfrac = (sxi >> 8) & 0xff; // ((short) sx) & 0xff;
+
+            // Put xfrac in mm5, yfrac in m6
+            pxor_r2r(mm5, mm5);
+            pxor_r2r(mm6, mm6);
+            movd_m2r(xfrac, mm5);
+            movd_m2r(yfrac, mm6);
+            punpcklwd_r2r(mm5, mm5);
+            punpcklwd_r2r(mm6, mm6);
+            punpckldq_r2r(mm5, mm5); /* 0X0X0X0X -> mm5 */                
+            punpckldq_r2r(mm6, mm6); /* 0Y0Y0Y0Y -> mm6 */
+            
+            // Load in the 4 bytes.
+            movd_m2r(*(unsigned int *) sp, mm1);
+            movd_m2r(*(unsigned int *) (sp + 4), mm2);
+            punpcklbw_r2r(mm7, mm1);            
+            punpcklbw_r2r(mm7, mm2);            
+
+            // Interpolate between a and b.
+            psubw_r2r(mm1, mm2);
+            pmullw_r2r(mm5, mm2);
+            psrlw_i2r(8, mm2);
+            paddb_r2r(mm2, mm1); /* mm1 contains I(a, b, xfrac); */
+
+            sp += srcpitch;
+            movd_m2r(*(unsigned int *) sp, mm3);
+            movd_m2r(*(unsigned int *) (sp + 4), mm2);
+
+            punpcklbw_r2r(mm7, mm3);            
+            punpcklbw_r2r(mm7, mm2);
+            
+            // Interpolate between c and d.
+            psubw_r2r(mm3, mm2);
+            pmullw_r2r(mm5, mm2);
+            psrlw_i2r(8, mm2);
+            paddb_r2r(mm2, mm3); /* mm3 contains I(c, d, xfrac); */
+
+            // Interpolate between ab and cd.
+            psubw_r2r(mm1, mm3);
+            pmullw_r2r(mm6, mm3);
+            psrlw_i2r(8, mm3);
+            paddb_r2r(mm3, mm1); /* mm1 contains I(ab, cd, yfrac) */
+
+            
+            // Store the result.
+            // packuswb_r2r(mm7, mm1);
+            // movd_r2m(mm1, *(unsigned int *)d);
+
+            // Alpha blend with dest.
+            movq_r2r(mm1, mm3);
+            psrlq_r2r(mm0, mm3); /* 000000AA -> m3 */
+
+            pmullw_r2r(mm4, mm3); // Alpha adjustment.
+            psrlw_i2r(8, mm3);
+
+            punpcklwd_r2r(mm3, mm3); /* 0000AAAA -> m3 */
+            movd_m2r(*(unsigned int *)d, mm2);
+            punpcklwd_r2r(mm3, mm3); /* AAAAAAAA -> m3 */
+            punpcklbw_r2r(mm7, mm2); /* a -> m2 */
+            psubw_r2r(mm2, mm1); /* b - a -> m1 */
+            pmullw_r2r(mm3, mm1); 
+            psrlw_i2r(8, mm1); /* alpha * (b-a) -> m1 */
+            paddb_r2r(mm2, mm1); /* a + alpha*(b-a) -> mm1 */
+
+            // Store the result.
+            packuswb_r2r(mm7, mm1);
+            movd_r2m(mm1, *(unsigned int *)d);
+            
+            d += 4;
+            sxi += xdxi;
+            syi += ydxi;
+        }
+
+        emms();
+    }
+
+    Py_END_ALLOW_THREADS
+
+
+        // This is bogus, and only serves to ensure that the FPU
+        // computes these variables at the right times.
+        return sxi + syi + xdxi + ydxi;
+}
+
+#endif
+
+void transform32_core(PyObject *pysrc, PyObject *pydst,
+                      float corner_x, float corner_y,
+                      float xdx, float ydx,
+                      float xdy, float ydy,
+                      int ashift,
+                      float a) {
+    
+#ifdef GCC_MMX
+    static int checked_mmx = 0;
+    static int has_mmx = 0;
+
+    if (! checked_mmx) {
+        has_mmx = SDL_HasMMX();
+        checked_mmx = 1;
+    }
+
+    if (has_mmx) {
+        transform32_mmx(pysrc, pydst, corner_x, corner_y,
+                        xdx, ydx, xdy, ydy, ashift, a);
+        return;
+    }
+    
+#endif
+    
+    transform32_std(pysrc, pydst, corner_x, corner_y,
+                    xdx, ydx, xdy, ydy, ashift, a);
+
+}
+                     
+
+
+                     
 void blend32_core_std(PyObject *pysrca, PyObject *pysrcb, PyObject *pydst,
                       int alpha) {
 
@@ -1555,3 +1764,46 @@ void colormatrix32_core(PyObject *pysrc, PyObject *pydst,
     Py_END_ALLOW_THREADS
 }
 
+void staticgray_core(PyObject *pysrc, PyObject *pydst,
+                     int rmul, int gmul, int bmul, int amul, int shift, char *vmap) {
+    
+    SDL_Surface *src;
+    SDL_Surface *dst;
+    
+    int srcpitch, dstpitch;
+    unsigned short dstw, dsth;
+    unsigned short x, y;
+
+    unsigned char *srcpixels;
+    unsigned char *dstpixels;
+
+    src = PySurface_AsSurface(pysrc);
+    dst = PySurface_AsSurface(pydst);
+    
+    Py_BEGIN_ALLOW_THREADS;
+
+    srcpixels = (unsigned char *) src->pixels;
+    dstpixels = (unsigned char *) dst->pixels;
+    srcpitch = src->pitch;
+    dstpitch = dst->pitch;
+
+    dstw = dst->w;
+    dsth = dst->h;
+
+    for (y = 0; y < dsth; y++) {
+        unsigned char *s = &srcpixels[y * srcpitch];
+        unsigned char *d = &dstpixels[y * dstpitch];
+
+        for (x = 0; x < dstw; x++) {
+            int sum = 0;
+
+            sum += *s++ * rmul;
+            sum += *s++ * gmul;
+            sum += *s++ * bmul;
+            sum += *s++ * amul;
+            *d++ = (unsigned char) vmap[sum >> shift];
+        }
+    }
+
+    Py_END_ALLOW_THREADS;
+}
