@@ -37,10 +37,24 @@ def path_to_common(renpy_base):
 # the path to a directory that will hold save files.
 def path_to_saves(gamedir):
     import renpy
-
+    
     if not renpy.config.save_directory:
         return gamedir + "/saves"
 
+    # Search the path above Ren'Py for a directory named "Ren'Py Data".
+    # If it exists, then use that for our save directory.
+    path = renpy.config.renpy_base
+
+    while True:
+        if os.path.isdir(path + "/Ren'Py Data"):
+            return path + "/Ren'Py Data/" + renpy.config.save_directory
+
+        newpath = os.path.dirname(path)
+        if path == newpath:
+            break
+        path = newpath
+
+    # Otherwise, put the saves in a standard place.
     if platform.mac_ver()[0]:
         rv = "~/Library/RenPy/" + renpy.config.save_directory
         return os.path.expanduser(rv)
@@ -48,7 +62,7 @@ def path_to_saves(gamedir):
         if 'APPDATA' in os.environ:
             return os.environ['APPDATA'] + "/RenPy/" + renpy.config.save_directory
         else:
-            rv = "/RenPy/" + renpy.config.save_directory
+            rv = "~/RenPy/" + renpy.config.save_directory
             return os.path.expanduser(rv)
     else:
         rv = "~/.renpy/" + renpy.config.save_directory
@@ -68,8 +82,8 @@ def path_to_renpy_base():
 ##############################################################################
 
 # The version of the Mac Launcher and py4renpy that we use.
-macos_version = (6, 7, 1)
-linux_version = (6, 7, 1)
+macos_version = (6, 9, 3)
+linux_version = (6, 9, 3)
 
 if __name__ == "__main__":
 
