@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -25,33 +25,27 @@
 # at least pcm_ok, we have no sound whatsoever.
 
 import renpy
+
+import pygame
 import os
 import atexit
+import time
 
 # Import the appropriate modules, or set them to None if we cannot.
 
 disable = os.environ.get("RENPY_DISABLE_SOUND", "")
 
 pss = None
-nativemidi = None
 mix = None
 
 if 'pss' not in disable:
     try:
         import pysdlsound as pss
-        pss.check_version(3)
+        pss.check_version(4)
         atexit.register(pss.quit)
     except:
         pss = None
 
-if 'nativemidi' not in disable:
-    try:
-        import nativemidi as nativemidi
-        atexit.register(nativemidi.stop)
-    except:
-        pass
-
-
 if 'mix' not in disable:
     try:
         import winmixer as mix
@@ -79,7 +73,6 @@ if mix:
             old_midi = mix.get_midi()
             mixer_enabled = True
             
-            
     def disable_mixer():
         global mixer_enabled
         
@@ -99,18 +92,21 @@ if mix:
 # This is True if we were able to sucessfully enable the pcm audio.
 pcm_ok = None
 
-# This is True if we were able to succesfully enable native midi.
-midi_ok = None
-
 # True if we are managing the mixers ourselves.
 mix_ok = None
 
-def ismidi(s):
+unique = time.time()
+serial = 0
+
+def get_serial():
     """
-    Returns true if s is the filename of a midi file.
+    Gets a globally unique serial number for each music change.
     """
     
-    return (midi_ok or renpy.config.reject_midi) and s is not None and s.endswith(".mid")
+    global serial
+    serial += 1
+    return (unique, serial)
+
 
 def load(fn):
     """
@@ -120,90 +116,6 @@ def load(fn):
     rv = renpy.loader.load(fn)
     return rv
 
-class Midi(object):
-    """
-    This is the object that manages the native midi playback. It ensures
-    that only one channel can play midi at a time, and it keeps track
-    of what's playing at any given time.
-    """
-
-    def __init__(self):
-
-        # The filename of the music that is currently playing through
-        # native midi.
-        self.playing = None
-
-        # The natural volume of the music that is playing through
-        # native midi.
-        self.volume = 1.0
-
-    def play(self, f, filename):
-
-        # Native midi must work for us to do anything.
-        if not midi_ok:
-            return
-
-        # If the midi channel is still busy, then ignore this request
-        # to play something new.
-        if self.playing:
-            return
-
-        # Otherwise, immediately play this midi file,
-        # replacing whatever is already playing.
-
-        try:
-            nativemidi.play(f)
-            self.playing = filename
-        except:
-            self.playing = None
-            if renpy.config.debug_sound:
-                raise
-
-    def busy(self):
-        """
-        Is midi busy?
-        """
-        
-        return self.playing is not None
-        
-    def periodic(self):
-        """
-        The 20hz callback that checks on the status of midi. This should 
-        be called before the updates for the various channel objects, below.
-        """
-
-        if not midi_ok:
-            return
-
-        if self.playing and not nativemidi.busy():
-            self.playing = None
-
-        # Eventually... add nativemidi fade code here.
-
-    def stop(self):
-        """
-        Called to stop the native midi subsystem from playing music.
-        """
-
-        if not midi_ok:
-            return
-
-        nativemidi.stop()
-        self.playing = None
-
-    def set_volume(self, volume):
-
-        if midi_ok:
-            nativemidi.set_volume(volume)            
-
-        self.volume = volume
-
-    def get_pos(self):
-
-        return nativemidi.get_pos()
-
-# A singleton Midi object that manages hardware midi playback.
-midi = Midi()
 
 class QueueEntry(object):
     """
@@ -216,11 +128,70 @@ class QueueEntry(object):
         self.tight = tight
 
 
-class Channel(object):
+class MusicContext(renpy.python.RevertableObject):
+    """
+    This stores information about the music in a game. This object
+    participates in rollback, so when the user goes back in time, all
+    the values get reverted as well.
+    """
 
-    def __init__(self, number):
-        # The number of this channel.
-        self.number = number
+    __version__ = 0
+
+    def __init__(self):
+
+        # The time this channel was last ordered panned.
+        self.pan_time = None
+
+        # The pan this channel was ordered to.
+        self.pan = 0
+
+        # The time the secondary volume was last ordered changed.
+        self.secondary_volume_time = None
+        
+        # The secondary volume.
+        self.secondary_volume = 1.0
+        
+        # The time the channel was ordered last changed.
+        self.last_changed = 0
+
+        # Was the last change tight?
+        self.last_tight = False
+
+        # What were the filenames we were ordered to loop last?
+        self.last_filenames = [ ]
+
+        # Should we force stop this channel?
+        self.force_stop = False
+        
+    def copy(self):
+        """
+        Returns a shallow copy of this context.
+        """
+
+        rv = MusicContext()
+        rv.__dict__.update(self.__dict__)
+
+        return rv
+        
+# The next channel number to be assigned.
+next_channel_number = 0
+        
+class Channel(object):
+    """
+    This stores information about the currently-playing music.
+    """
+    
+    def __init__(self, name, default_loop, stop_on_mute, tight):
+
+        # The name assigned to this channel. This is used to look up
+        # information about the channel in the MusicContext object.
+        self.name = name
+        
+        # The number this channel has been assigned, or None if we've yet
+        # to assign a number to the channel. We only assign a channel
+        # number when there's an operation on the channel other than
+        # setting the mixer.
+        self._number = None
 
         # The name of the mixer this channel uses. Set below, as there's
         # no good default.
@@ -243,10 +214,6 @@ class Channel(object):
         # Are we playing anything at all?
         self.playing = False
 
-        # If True, then this channel is playing midi. If False, it's
-        # playing PCM instead.
-        self.playing_midi = False
-
         # If True, we'll wait for this channel to stop before
         # loading in more music from the queue. (This is necessary to
         # do a synchro-start.)
@@ -256,13 +223,73 @@ class Channel(object):
         # once all channels are ready.
         self.synchro_start = False
 
-        # If we're a music channel, the time the music in this channel was
-        # last changed.
-        self.music_last_changed = 0
+        # The time the music in this channel was last changed.
+        self.last_changed = 0
 
         # The callback that is called if the queue becomes empty.
         self.callback = None
 
+        # The time this channel was last panned.
+        self.pan_time = None
+
+        # The time the secondary volume of this channel was last set.
+        self.secondary_volume_time = None
+
+        # Should we stop playing on mute?
+        self.stop_on_mute = stop_on_mute
+
+        # Is this channel tight?
+        self.tight = tight
+
+        # The number of items in the queue that should be kept
+        # on queue clear.
+        self.keep_queue = 0
+
+        
+        if default_loop is None:
+            # By default, should we loop the music?
+            self.default_loop = True
+            # Was this set explicitly?
+            self.default_loop_set = False
+
+        else:
+            self.default_loop = default_loop
+            self.default_loop_set = True
+        
+
+    def get_number(self):
+        """
+        Returns the number of this channel, allocating a number if that
+        proves necessary.
+        """
+        global next_channel_number
+        
+        rv = self._number
+        if rv is None:
+            rv = self._number = next_channel_number
+            next_channel_number += 1
+        
+        return rv
+
+    number = property(get_number)
+
+    def get_context(self):
+        """
+        Returns the MusicContext corresponding to this channel, taken from
+        the context object. Allocates a MusicContext if none exists.
+        """
+
+        mcd = renpy.game.context().music
+
+        rv = mcd.get(self.name)
+        if rv is None:
+           rv = mcd[self.name] = MusicContext();
+
+        return rv
+
+    context = property(get_context)
+        
+    
     def periodic(self):
         """
         This is the periodic call that causes this channel to load new stuff
@@ -271,20 +298,12 @@ class Channel(object):
 
         # This should be set from something that checks to see if our
         # mixer is muted.
-        force_stop = renpy.game.preferences.mute[self.mixer]
+        force_stop = self.context.force_stop or (renpy.game.preferences.mute[self.mixer] and self.stop_on_mute)
 
         if self.playing and force_stop:
-
-            if self.playing_midi:
-                midi.stop()
-                self.playing = False
-                self.wait_stop = False
-                self.playing_midi = False
-
-            else:
-                pss.stop(self.number)
-                self.playing = False
-                self.wait_stop = False
+            pss.stop(self.number)
+            self.playing = False
+            self.wait_stop = False
 
         if force_stop:
             if self.loop:
@@ -292,26 +311,18 @@ class Channel(object):
             else:
                 self.queue = [ ]
             return
-
-        # If we're playing midi, and the midi device is busy, return.
-        # Otherwise, it's stopped, so we can stop waiting.
-        if self.playing_midi:
-            if midi.busy():
-                return
-            else:
-                self.playing = False
-                self.wait_stop = False
-                self.playing_midi = False
-
-
+        
         # Should we do the callback?
         do_callback = False
 
+        # This has been modified so we only queue a single sound file
+        # per call, to prevent memory leaks with really short sound
+        # files. So this loop will only execute once, in practice.
         while True:
 
             depth = pss.queue_depth(self.number)
 
-            if depth == 0 and not self.playing_midi:
+            if depth == 0:
                 self.wait_stop = False
                 self.playing = False
 
@@ -319,11 +330,6 @@ class Channel(object):
             if not self.queue:
                 break
 
-            # If we're still playing midi, then we can't queue
-            # anything up, so bail out now.
-            if self.playing_midi:
-                break
-
             # If the pcm_queue is full, then we can't queue
             # anything, regardless of if it is midi or pcm.
             if depth >= 2:
@@ -339,48 +345,27 @@ class Channel(object):
                 break
 
             # Otherwise, we might be able to enqueue something.
-            topq = self.queue[0]
+            topq = self.queue.pop(0)
 
-            # We can only play midi if we're not playing PCM.
-            if ismidi(topq.filename):
-                if depth != 0:
-                    break
+            # Blacklist of old file formats we used to support, but we now
+            # ignore.
+            lfn = topq.filename.lower()
+            for i in (".mod", ".xm", ".mid", ".midi"):
+                if lfn.endswith(i):
+                    topq = None
 
-            # if 0 <= self.get_pos() <= 1000:
-            #    break
+            if not topq:
+                continue
             
-            # At this point, we've decided to try to play
-            # top. So let's see how far we can get.
-
-            # Update the queue
-            self.queue = self.queue[1:]
-
             try:
                 topf = load(topq.filename)
 
-                if ismidi(topq.filename):
-
-                    if renpy.config.reject_midi:
-                        raise Exception("Midi files are no longer supported.")
-                    
-                    # If someone else is playing midi, then raise
-                    # an error to that effect.
-                    if midi.busy():
-                        raise Exception("We can only play one midi at a time.")
-
-                    # Play midi.
-                    midi.play(topf, topq.filename)
-                    self.playing = True
-                    self.playing_midi = True
-
+                if depth == 0:
+                    pss.play(self.number, topf, topq.filename, paused=self.synchro_start, fadein=topq.fadein, tight=topq.tight)
                 else:
+                    pss.queue(self.number, topf, topq.filename, fadein=topq.fadein, tight=topq.tight)
 
-                    if depth == 0:
-                        pss.play(self.number, topf, topq.filename, paused=self.synchro_start, fadein=topq.fadein, tight=topq.tight)
-                    else:
-                        pss.queue(self.number, topf, topq.filename, fadein=topq.fadein, tight=topq.tight)
-
-                    self.playing = True
+                self.playing = True
 
             except:
                 if renpy.config.debug_sound:
@@ -388,38 +373,60 @@ class Channel(object):
                 else:
                     return
 
-            if self.loop and not self.queue:
-                for i in self.loop:
-                    newq = QueueEntry(i, 0, topq.tight)                    
-                    self.queue.append(newq)
-            else:
-                do_callback = True
+            break
 
-        # TODO: Queue empty callback.
+        if self.loop and not self.queue:
+            for i in self.loop:
+                newq = QueueEntry(i, 0, topq.tight)                    
+                self.queue.append(newq)
+        else:
+            do_callback = True
+                
+        # Queue empty callback.
         if do_callback and self.callback:
             self.callback()
 
     def dequeue(self, even_tight=False):
         """
-        Clears any queued music. Doesn't stop the playing music, if
-        any, but will prevent looping from occuring.
+        Clears the queued music.
+
+        If the first item in the queue has not been started, then it is
+        left in the queue unless all is given.
         """
 
-        self.queue = [ ]
+        self.queue = self.queue[:self.keep_queue]
         self.loop = [ ]
-
+            
         if not pcm_ok:
             return
 
-        if not self.playing_midi:
+        if self.keep_queue == 0:
             pss.dequeue(self.number, even_tight)
 
     def interact(self):
         """
-        Called (mostly) once per interaction. Calls the queue callback
-        if it's becoming empty.
+        Called (mostly) once per interaction.
         """
 
+        self.keep_queue = 0
+        
+        if pcm_ok:
+
+            if self.pan_time != self.context.pan_time:
+                self.pan_time = self.context.pan_time
+                pss.set_pan(self.number,
+                            self.context.pan,
+                            0)
+
+
+            if self.secondary_volume_time != self.context.secondary_volume_time:
+                self.secondary_volume_time = self.context.secondary_volume_time
+                pss.set_secondary_volume(self.number,
+                                         self.context.secondary_volume,
+                                         0)
+                
+        self.pan_delay = 0
+            
         if not self.queue and self.callback:
             self.callback()
 
@@ -430,18 +437,19 @@ class Channel(object):
         of seconds. Also clears any queued music.
         """
 
+        self.keep_queue = 0
         self.dequeue()
 
         if not pcm_ok:
             return
 
-        if self.playing_midi:
-            midi.stop()
-        else:
+        if secs == 0:
+            pss.stop(self.number)
+        else:            
             pss.fadeout(self.number, int(secs * 1000))
 
 
-    def enqueue(self, filenames, loop=True, synchro_start=False, fadein=0, tight=False):
+    def enqueue(self, filenames, loop=True, synchro_start=False, fadein=0, tight=None):
 
         for filename in filenames:
             renpy.game.persistent._seen_audio[filename] = True
@@ -449,10 +457,18 @@ class Channel(object):
         if not pcm_ok:
             return
 
+        if tight is None:
+            tight = self.tight
+
+        self.keep_queue += 1
+            
         for filename in filenames:
             qe = QueueEntry(filename, int(fadein * 1000), tight)
             self.queue.append(qe)
 
+            # Only fade the first thing in.
+            fadein = 0
+
         if loop:
             self.loop = list(filenames)
         else:
@@ -466,10 +482,7 @@ class Channel(object):
         if not pcm_ok:
             return None
 
-        if self.playing_midi:
-            return midi.playing
-        else:
-            return pss.playing_name(self.number)
+        return pss.playing_name(self.number)
 
     def set_volume(self, volume):
         self.chan_volume = volume
@@ -479,34 +492,67 @@ class Channel(object):
         if not pcm_ok:
             return -1
         
-        if self.playing_midi:
-            return midi.get_pos()
-        else:
-            return pss.get_pos(self.number)
+        return pss.get_pos(self.number)
 
-            
-# The number of channels we support.
-NUM_CHANNELS = 8
+    def set_pan(self, pan, delay):
+        now = get_serial()
+        self.context.pan_time = now
+        self.context.pan = pan
 
-# A list of channels.
-channels = [ Channel(i) for i in range(NUM_CHANNELS) ]
+        if pcm_ok:
+            self.pan_time = self.context.pan_time
+            pss.set_pan(self.number, self.context.pan, delay)
+
+    def set_secondary_volume(self, volume, delay):
+        now = get_serial()
+        self.context.secondary_volume_time = now
+        self.context.secondary_volume = volume
+
+        if pcm_ok:
+            self.secondary_volume_time = self.context.secondary_volume_time
+            pss.set_secondary_volume(self.number, self.context.secondary_volume, delay)
+
+# A list of channels we know about.
+all_channels = [ ]
+
+# A map from channel name to Channel object.
+channels = { }
+
+
+def register_channel(name, mixer=None, loop=None, stop_on_mute=True, tight=False):
+    if not renpy.game.init_phase:
+        raise Exception("Can't register channel outside of init phase.")
+
+    c = Channel(name, loop, stop_on_mute, tight)
+    c.mixer = mixer
+    all_channels.append(c)
+    channels[name] = c
     
-def get_channel(number):
-    if not 0 <= number < NUM_CHANNELS:
-        raise Exception("Channel number %d out of bounds." % channel)
+def alias_channel(name, newname):
+    if not renpy.game.init_phase:
+        raise Exception("Can't alias channel outside of init phase.")
 
-    return channels[number]
+    c = get_channel(name)
+    channels[newname] = c
     
+def get_channel(name):
+
+    rv = channels.get(name)
+    if rv is None:
+        raise Exception("Audio channel %r is unknown." % name)
+        
+    return rv
+
+def set_force_stop(name, value):
+    get_channel(name).context.force_stop = value
 
 def init():
 
     global pcm_ok
-    global midi_ok
     global mix_ok
 
     if not renpy.config.sound:
         pcm_ok = False
-        midi_ok = False
         mix_ok = False
         return
 
@@ -517,30 +563,17 @@ def init():
             bufsize = int(os.environ['RENPY_SOUND_BUFSIZE'])
 
         try:
-            pss.init(renpy.config.sound_sample_rate, 2, bufsize)
+            pss.init(renpy.config.sound_sample_rate, 2, bufsize, renpy.config.debug_sound)
             pcm_ok = True
         except:
             if renpy.config.debug_sound:
                 raise
             pcm_ok = False
 
-    if renpy.config.reject_midi:
-        midi_ok = False
-            
-    if midi_ok is None and nativemidi:
-
-        try:
-            nativemidi.init()
-            midi_ok = True
-        except:
-            if renpy.config.debug_sound:
-                raise
-            midi_ok = False
-
     # Find all of the mixers in the game.
     mixers = [ ]
 
-    for c in channels:
+    for c in all_channels:
         if c.mixer not in mixers:
             mixers.append(c.mixer)
 
@@ -549,7 +582,6 @@ def init():
     if mix and not 'RENPY_NOMIXER' in os.environ and mix.get_wave() is not None:
         default_volume = mix.get_wave()
         mix_ok = True
-
     
     for m in mixers:
         renpy.game.preferences.volumes.setdefault(m, default_volume)
@@ -565,7 +597,7 @@ def quit():
     if not pcm_ok:
         return
     
-    for c in channels:
+    for c in all_channels:
         c.dequeue()
         c.fadeout(0)
         
@@ -578,15 +610,12 @@ def quit():
         
     pss.quit()
     
-    pcm_ok = None    
-    midi_ok = None
+    pcm_ok = None
     mix_ok = None
  
 # The last-set pcm volume.
 pcm_volume = None
 
-# The last-set midi volume.
-midi_volume = None
 
 def periodic():
     """
@@ -597,22 +626,21 @@ def periodic():
     """
 
     global pcm_volume
-    global midi_volume
 
     if not pcm_ok:
         return False
 
     try:
 
-        for c in channels:
+        for c in all_channels:
             c.periodic()
 
-        midi.periodic()
-
+        pss.periodic()
+        
         # Perform a synchro-start if necessary.
         need_ss = False
 
-        for c in channels:
+        for c in all_channels:
             
             if c.synchro_start and c.wait_stop:
                 need_ss = False
@@ -624,14 +652,14 @@ def periodic():
         if need_ss:
             pss.unpause_all()
 
-            for c in channels:
+            for c in all_channels:
                 c.synchro_start = False
 
         # Now, consider adjusting the volume of the channel. 
 
         max_volume = -1.0
         volumes = renpy.game.preferences.volumes
-        
+
         if mix_ok:
 
             anything_playing = False
@@ -643,63 +671,45 @@ def periodic():
                 if vol != 0:
                     anything_playing = True
 
-            if not anything_playing:
-                disable_mixer()
-                return
-
-            enable_mixer()
-                
             if max_volume == -1.0:
                 return
 
-            if max_volume == 0.0:
-                return
+            if not anything_playing:
+                disable_mixer()
+                pcm_volume = -1.0
+            else:
+                enable_mixer()
+                
+                if max_volume != pcm_volume:    
+                    mix.set_wave(max_volume)
+                    pcm_volume = max_volume
 
-            if max_volume != pcm_volume:    
-                mix.set_wave(max_volume)
-                pcm_volume = max_volume
 
-            for c in channels:
-
-                # if not c.playing:
-                #    continue
+            for c in all_channels:
 
                 vol = c.chan_volume * volumes[c.mixer]
 
-                if c.playing_midi:
-                    if midi_volume != vol:
-                        # mix.set_midi(vol)
-                        midi.set_volume(vol)
-                        midi_volume = vol
-                else:
-                    vol /= max_volume
-
-                    if c.actual_volume != vol:
-                        pss.set_volume(c.number, vol)
-                        c.actual_volume = vol
+                vol /= max_volume
+                
+                if c.actual_volume != vol:
+                    pss.set_volume(c.number, vol)
+                    c.actual_volume = vol
 
         else:
 
-            for c in channels:
-
-                if not c.playing:
-                    continue
+            for c in all_channels:
 
                 vol = c.chan_volume * volumes[c.mixer]
 
-                if c.playing_midi:
-                    if vol != midi_volume:
-                        midi.set_volume(vol)
-                        midi_volume = vol
-                else:
-                    if vol != c.actual_volume:
-                        pss.set_volume(c.number, vol)
-                        c.actual_volume = vol
-
+                if vol != c.actual_volume:
+                    pss.set_volume(c.number, vol)
+                    c.actual_volume = vol
+                    
     except:
         if renpy.config.debug_sound:
             raise
 
+        
 def interact():
     """
     Called at least once per interaction.
@@ -709,13 +719,70 @@ def interact():
         return
 
     try:
-        for c in channels:
+        for c in all_channels:
+
             c.interact()
 
+            # if _music_volumes.get(i, 1.0) != c.chan_volume:
+            #    c.set_volume(_music_volumes.get(i, 1.0))
+
+            ctx = c.context
+                
+            # If we're in the same music change, then do nothing with the
+            # music.
+            if c.last_changed == ctx.last_changed:
+                continue
+
+            filenames = ctx.last_filenames
+            tight = ctx.last_tight
+
+            if c.default_loop:
+                if not filenames or c.get_playing() not in filenames:
+                    c.fadeout(renpy.config.fade_music)
+
+            if filenames:
+                c.enqueue(filenames, loop=True, synchro_start=True, tight=tight)
+
+            c.last_changed = ctx.last_changed
+
     except:
         if renpy.config.debug_sound:
             raise
 
-    renpy.audio.music.interact()
     periodic()
 
+
+ALLOC_EVENT = pygame.USEREVENT
+REFRESH_EVENT = pygame.USEREVENT + 1
+    
+def event(ev):
+    """
+    Handles an event generated by pss (really, the ffdecode
+    module). Returns true if we've handled it, or false if we should
+    be pushing it to another module for further handling.
+    """
+
+    if not pcm_ok:
+        return False
+
+    
+    if ev.type == ALLOC_EVENT:
+        if renpy.display.video.fullscreen or not renpy.display.video.surface:
+            surf = pygame.display.get_surface()
+        else:
+            surf = renpy.display.video.surface
+
+        pss.alloc_event(renpy.display.scale.real(surf))
+            
+        return True
+
+    if ev.type == REFRESH_EVENT:
+        if renpy.audio.music.get_playing("movie"):
+            pss.refresh_event()
+            
+        # Return False, as a Movie should get this to know when to
+        # redraw itself.
+        return False
+        
+    return False
+    
