@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -25,7 +25,6 @@
 
 # This will be deleted by the end of this file.
 import renpy
-import sets
 import os
 
 # Can we add more config variables?
@@ -40,6 +39,9 @@ window_title = "A Ren'Py Game"
 # An image file containing the window icon image.
 window_icon = None
 
+# The same, but only used on MS windows.
+windows_icon = None
+
 # The width and height of the drawable area of the screen.
 screen_width = 800
 screen_height = 600
@@ -165,7 +167,7 @@ keymap = dict(
     # disabled.
     rollback = [ 'K_PAGEUP', 'mousedown_4', 'joy_rollback' ],
     screenshot = [ 's' ],
-    toggle_fullscreen = [ 'f' ],
+    toggle_fullscreen = [ 'f', 'alt_K_RETURN', 'alt_K_KP_ENTER' ],
     toggle_music = [ 'm' ],
     game_menu = [ 'K_ESCAPE', 'mouseup_3', 'joy_menu' ],
     hide_windows = [ 'mouseup_2', 'h', 'joy_hide' ],
@@ -174,6 +176,9 @@ keymap = dict(
     reload_game = [ 'R' ],
     inspector = [ 'I' ],
     developer = [ 'D' ],
+    quit = [ 'meta_q', 'alt_K_F4', 'alt_q' ],
+    iconify = [ 'meta_m', 'alt_m' ],
+    help = [ 'K_F1', 'meta_shift_/' ],
     
     # Say.
     rollforward = [ 'mousedown_5', 'K_PAGEDOWN' ],
@@ -260,7 +265,7 @@ font_replacement_map = { }
 with_callback = None
 
 # The framerate limit, in frames per second.
-framerate = 60
+framerate = 100
 
 # The number of frames that Ren'Py has shown.
 frames = 0
@@ -273,6 +278,9 @@ editor = os.environ.get('RENPY_EDITOR', None)
 # for example, to display traceback.txt.
 editor_transient = os.environ.get('RENPY_EDITOR_TRANSIENT', editor)
 
+# The separator used between files in the text editor.
+editor_file_separator = os.environ.get('RENPY_EDITOR_FILE_SEPARATOR', '" "')
+
 # Enable developer mode?
 developer = False
 
@@ -369,8 +377,18 @@ auto_save_extra_info = None
 # game-specific data is saved.
 save_directory = None
 
+# These are used to deal with the case where a picture is missing.
+missing_scene = None
+missing_show = None
+missing_hide = None
+
+# This is called when control is transferred to a label.
+label_callback = None
+
+# A function that is called when the window needs to be shown.
+empty_window = None
+
 del renpy
-del sets
 
 def init():
     import renpy
