@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -84,6 +84,12 @@ def compile_event(key, keydown):
     else:
         rv += " and not (ev.mod & %d)" % pygame.KMOD_ALT
 
+    if part[0] == "meta":
+        part.pop(0)
+        rv += " and (ev.mod & %d)" % pygame.KMOD_META
+    else:
+        rv += " and not (ev.mod & %d)" % pygame.KMOD_META
+
     if part[0] == "shift":
         part.pop(0)
         rv += " and (ev.mod & %d)" % pygame.KMOD_SHIFT
@@ -181,9 +187,7 @@ class Keymap(renpy.display.layout.Null):
 
 class RollForward(renpy.display.layout.Null):
     """
-    This is a behavior that maps keys to functions that are called when
-    the key is pressed. The keys are specified by giving the appropriate
-    k_constant from pygame.constants, or the unicode for the key.
+    This behavior implements rollforward.
     """
 
     def __init__(self, value):
@@ -196,6 +200,7 @@ class RollForward(renpy.display.layout.Null):
         if map_event(ev, "rollforward"):
             renpy.game.interface.suppress_transition = True
             renpy.game.after_rollback = True
+            renpy.game.log.rolled_forward = True
             return self.value
 
 class PauseBehavior(renpy.display.layout.Null):
@@ -218,6 +223,27 @@ class PauseBehavior(renpy.display.layout.Null):
 
         renpy.game.interface.timeout(self.delay - st)
 
+class SoundStopBehavior(renpy.display.layout.Null):
+    """
+    This is a class implementing the sound stop behavior,
+    which is to return False when a sound is no longer playing
+    on the named channel.
+    """
+
+    def __init__(self, channel, result=False):
+        super(SoundStopBehavior, self).__init__()
+
+        self.channel = channel
+        self.result = result 
+
+
+    def event(self, ev, x, y, st):
+        
+        if not renpy.audio.music.get_playing(self.channel):
+            return self.result
+
+        renpy.game.interface.timeout(.025)
+
 class SayBehavior(renpy.display.layout.Null):
     """
     This is a class that implements the say behavior,
@@ -249,22 +275,6 @@ class SayBehavior(renpy.display.layout.Null):
               
     def event(self, ev, x, y, st):
 
-        skip_delay = renpy.config.skip_delay / 1000.0
-
-        if renpy.config.allow_skipping and renpy.config.skipping and \
-           st >= skip_delay:
-
-            if renpy.game.preferences.skip_unseen:
-                return True
-            elif renpy.config.skipping == "fast":
-                return True
-            elif renpy.game.context().seen_current(True):
-                return True
-
-        if renpy.config.allow_skipping and renpy.config.skipping and \
-           st < skip_delay:
-            renpy.game.interface.timeout(skip_delay - st)
-
         if self.afm_length and renpy.game.preferences.afm_time and renpy.game.preferences.afm_enable:
                                                           
             afm_delay = ( 1.0 * ( renpy.config.afm_bonus + self.afm_length ) / renpy.config.afm_characters ) * renpy.game.preferences.afm_time
@@ -284,7 +294,7 @@ class SayBehavior(renpy.display.layout.Null):
                 renpy.game.interface.timeout(afm_delay - st)
 
         for dismiss in self.dismiss:
-
+            
             if map_event(ev, dismiss) and self.is_focused():
 
                 if renpy.config.skipping:
@@ -298,7 +308,25 @@ class SayBehavior(renpy.display.layout.Null):
 
                 return True
             
+        skip_delay = renpy.config.skip_delay / 1000.0
+
+        if renpy.config.allow_skipping and renpy.config.skipping and \
+           st >= skip_delay:
+
+            if renpy.game.preferences.skip_unseen:
+                return True
+            elif renpy.config.skipping == "fast":
+                return True
+            elif renpy.game.context().seen_current(True):
+                return True
+
+        if renpy.config.allow_skipping and renpy.config.skipping and \
+           st < skip_delay:
+            renpy.game.interface.timeout(skip_delay - st)
+
+
         return None
+
     
 class Button(renpy.display.layout.Window):
 
@@ -395,8 +423,9 @@ class Button(renpy.display.layout.Window):
 
             self.activated = True
             self.style.set_prefix(self.role + 'activate_')
-            
-            renpy.audio.sound.play(self.style.sound)
+
+            if self.style.sound:
+                renpy.audio.music.play(self.style.sound, channel="sound")
                     
             rv = self.clicked()
 
@@ -618,6 +647,15 @@ class Bar(renpy.display.core.Displayable):
         value = self.adjustment.value
         page = self.adjustment.page
 
+        if range <= 0:
+            if self.style.unscrollable == "hide":
+                self.hidden = True
+                return renpy.display.render.Render(width, height)
+            elif self.style.unscrollable == "insensitive":
+                self.set_style_prefix("insensitive_")
+
+        self.hidden = False
+        
         if self.style.bar_invert ^ self.style.bar_vertical:
             value = range - value
 
@@ -712,11 +750,14 @@ class Bar(renpy.display.core.Displayable):
     def event(self, ev, x, y, st):
 
         if not self.focusable:
-            return
+            return None
 
         if not self.is_focused():
-            return
+            return None
 
+        if self.hidden:
+            return None
+        
         range = self.adjustment.range
         old_value = self.adjustment.value
         value = old_value
