@@ -497,7 +497,7 @@ class SceneLists(renpy.object.Object):
 
         if key and name:
             self.image_predict_info.images[layer][key] = name
-                
+
         l = self.layers[layer]
 
         at = None
@@ -1145,6 +1145,8 @@ class Interface(object):
                     yanchor = 0,
                     xmaximum = w,
                     ymaximum = h,
+                    xminimum = w,
+                    yminimum = h,
                     clipping = True,
                     )
 
