@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -161,7 +161,7 @@ class Cache(object):
 
         if not image.cache:
             surf = image.load()
-            surf = surf.convert_alpha()
+            surf = surf.convert_alpha(renpy.game.interface.display.window)
                 
             renpy.display.render.mutated_surface(surf)
             return surf
@@ -187,8 +187,8 @@ class Cache(object):
             
             surf = image.load()
             has_alpha = surf.get_masks()[3]
-            surf = surf.convert_alpha()
-            
+            surf = surf.convert_alpha(renpy.game.interface.display.window)
+
             ce = CacheEntry(image, surf)
             self.total_cache_size += ce.size
             self.cache[image] = ce
@@ -202,26 +202,27 @@ class Cache(object):
             # RLE detection. (ce.size is used to check that we're not
             # 0 pixels big.)
             if id(ce.surf) not in rle_cache and ce.size:
-                rle = image.rle
+                rle = not renpy.game.less_memory
+                # rle = image.rle
                 surf = ce.surf
 
-                # If we don't know if the image is RLE or not, guess.
-                # Only do so if the image has an alpha channel.
-                if rle is None and has_alpha:
-                    sw, sh = surf.get_size()
-
-                    for i in range(0, 10):
-                        if surf.get_at((random.randint(0, sw-1),
-                                        random.randint(0, sh-1)))[3] == 0:
-                            rle = True
-                            break
+#                 # If we don't know if the image is RLE or not, guess.
+#                 # Only do so if the image has an alpha channel.
+#                 if rle is None and has_alpha and not renpy.game.less_memory:
+#                     sw, sh = surf.get_size()
 
+#                     for i in range(0, 10):
+#                         if surf.get_at((random.randint(0, sw-1),
+#                                         random.randint(0, sh-1)))[3] == 0:
+#                             rle = True
+#                             break
+                
                 if rle:
-
+                    
                     # We must copy the surface, so we have a RLE-specific version.
-                    rle_surf = ce.surf.convert_alpha()
-
+                    rle_surf = ce.surf.convert_alpha(renpy.game.interface.display.window)
                     rle_surf.set_alpha(255, RLEACCEL)
+
                     rle_cache[id(ce.surf)] = rle_surf
                     renpy.display.render.mutated_surface(ce.surf)
                     if renpy.config.debug_image_cache:
@@ -479,7 +480,7 @@ class Image(ImageBase):
                 if im is not None:
                     return im.predict_files()
 
-            return [ ]
+            return [ self.filename ]
 
 class ZipFileImage(ImageBase):
 
@@ -621,11 +622,16 @@ class FrameImage(ImageBase):
         
         xb = renpy.display.scale.scale(self.xborder)
         yb = renpy.display.scale.scale(self.yborder)
-        
+
         sw, sh = source.get_size()
         dw, dh = dest.get_size()
-    
 
+        if xb * 2 >= sw:
+            xb = sw / 2 - 1
+
+        if yb * 2 >= sh:
+            yb = sh / 2 - 1
+        
         def draw(x0, x1, y0, y1):
 
             # Compute the coordinates of the left, right, top, and
@@ -670,7 +676,7 @@ class FrameImage(ImageBase):
 
             # Compute sizes.
             srcsize = (sx1 - sx0, sy1 - sy0)
-            dstsize = (dx1 - dx0, dy1 - dy0)
+            dstsize = (int(dx1 - dx0), int(dy1 - dy0))
 
             # Get a subsurface.
             surf = source.subsurface((sx0, sy0, srcsize[0], srcsize[1]))
@@ -681,7 +687,7 @@ class FrameImage(ImageBase):
                     tilew, tileh = srcsize
                     dstw, dsth = dstsize
 
-                    surf2 = pygame.Surface(dstsize, 0, surf)
+                    surf2 = renpy.display.scale.PygameSurface(dstsize, 0, surf)
 
                     for y in range(0, dsth, tileh):
                         for x in range(0, dstw, tilew):
@@ -701,6 +707,7 @@ class FrameImage(ImageBase):
             # Blit.
             dest.blit(surf, (dx0, dy0))
 
+
         # Top row.
         draw(0, xb, 0, yb)
         draw(xb, -xb, 0, yb)
@@ -715,7 +722,7 @@ class FrameImage(ImageBase):
         draw(0, xb, -yb, 0)
         draw(xb, -xb, -yb, 0)
         draw(-xb, 0, -yb, 0)
-        
+
         # And, finish up.
         return rv
 
@@ -757,6 +764,47 @@ class Scale(ImageBase):
         im = image(im)
         super(Scale, self).__init__(im, width, height, bilinear, **properties)
 
+        self.image = im
+        self.width = int(width)
+        self.height = int(height)
+        self.bilinear = bilinear
+
+    def load(self):
+
+        child = cache.get(self.image)
+        
+        if self.bilinear:
+            try:
+                renpy.display.render.blit_lock.acquire()
+                rv = renpy.display.scale.smoothscale(child, (self.width, self.height))
+            finally:
+                renpy.display.render.blit_lock.release()
+        else:
+            try:
+                renpy.display.render.blit_lock.acquire()
+                rv = pygame.transform.scale(child, (self.width, self.height))
+            finally:
+                renpy.display.render.blit_lock.release()
+            
+        return rv
+
+    def predict_files(self):
+        return self.image.predict_files()
+
+class FactorScale(ImageBase):
+    """
+    This is the same, but it takes a factor rather than an absolute scale
+    size.
+    """
+
+    def __init__(self, im, width, height=None, bilinear=True, **properties):
+
+        if height is None:
+            height = width
+        
+        im = image(im)
+        super(FactorScale, self).__init__(im, width, height, bilinear, **properties)
+
         self.image = im
         self.width = width
         self.height = height
@@ -764,15 +812,25 @@ class Scale(ImageBase):
 
     def load(self):
 
+        surf = cache.get(self.image)
+        width, height = surf.get_size()
+
+        width = int(width * self.width)
+        height = int(height * self.height)
+
         if self.bilinear:
-            surf = cache.get(self.image)
-            rv = pygame.Surface((self.width, self.height), 0, surf)
-            renpy.display.module.bilinear_scale(surf, rv)
+            try:
+                renpy.display.render.blit_lock.acquire()
+                rv = renpy.display.scale.smoothscale(surf, (width, height))
+            finally:
+                renpy.display.render.blit_lock.release()
 
         else:
-            renpy.display.render.blit_lock.acquire()
-            rv = pygame.transform.scale(cache.get(self.image), (self.width, self.height))
-            renpy.display.render.blit_lock.release()
+            try:
+                renpy.display.render.blit_lock.acquire()
+                rv = pygame.transform.scale(surf, (width, height))
+            finally:
+                renpy.display.render.blit_lock.release()
             
         return rv
 
@@ -808,9 +866,13 @@ class Flip(ImageBase):
 
     def load(self):
 
-        renpy.display.render.blit_lock.acquire()
-        rv = pygame.transform.flip(cache.get(self.image), self.horizontal, self.vertical)
-        renpy.display.render.blit_lock.release()
+        child = cache.get(self.image)
+        
+        try:
+            renpy.display.render.blit_lock.acquire()
+            rv = pygame.transform.flip(child, self.horizontal, self.vertical)
+        finally:
+            renpy.display.render.blit_lock.release()
 
         return rv
 
@@ -845,9 +907,13 @@ class Rotozoom(ImageBase):
 
     def load(self):
 
-        renpy.display.render.blit_lock.acquire()
-        rv = pygame.transform.rotozoom(cache.get(self.image), self.angle, self.zoom)
-        renpy.display.render.blit_lock.release()
+        child = cache.get(self.image)
+        
+        try:
+            renpy.display.render.blit_lock.acquire()
+            rv = pygame.transform.rotozoom(child, self.angle, self.zoom)
+        finally:
+            renpy.display.render.blit_lock.release()
 
         return rv
 
@@ -938,7 +1004,7 @@ class Map(ImageBase):
             return surf
 
         if self.force_alpha and not (surf.get_masks()[3]):
-            surf = surf.convert_alpha()
+            surf = surf.convert_alpha(renpy.game.interface.display.window)
 
         rv = pygame.Surface(surf.get_size(), surf.get_flags(), surf)
 
@@ -983,7 +1049,7 @@ class Twocolor(ImageBase):
             return surf
 
         if self.force_alpha and not (surf.get_masks()[3]):
-            surf = surf.convert_alpha()
+            surf = surf.convert_alpha(renpy.game.interface.display.window)
 
         rv = pygame.Surface(surf.get_size(), surf.get_flags(), surf)
 
@@ -1026,7 +1092,7 @@ class Recolor(ImageBase):
             return surf
 
         if self.force_alpha and not (surf.get_masks()[3]):
-            surf = surf.convert_alpha()
+            surf = surf.convert_alpha(renpy.game.interface.display.window)
 
         rv = pygame.Surface(surf.get_size(), surf.get_flags(), surf)
 
@@ -1065,7 +1131,7 @@ class MatrixColor(ImageBase):
             return surf
 
         if not (surf.get_masks()[3]):
-            surf = surf.convert_alpha()
+            surf = surf.convert_alpha(renpy.game.interface.display.window)
 
         rv = pygame.Surface(surf.get_size(), surf.get_flags(), surf)
 
@@ -1223,6 +1289,8 @@ im.matrix(%f, %f, %f, %f, %f.
     def contrast(c):
         return matrix.brightness(-.5) * matrix.tint(c, c, c) * matrix.brightness(.5) 
 
+    contrast = staticmethod(contrast)
+    
 
     # from http://www.gskinner.com/blog/archives/2005/09/flash_8_source.html
     def hue(h):
@@ -1241,14 +1309,16 @@ im.matrix(%f, %f, %f, %f, %f.
             )
 
     hue = staticmethod(hue)
-
     
-def Grayscale(im, desat=(0.2126, 0.7152, 0.0722)):
-    return MatrixColor(im, matrix.saturation(0.0, desat))
 
-def Sepia(im, tint=(1.0, .94, .76), desat=(0.2126, 0.7152, 0.0722)):
-    return MatrixColor(im, matrix.saturation(0.0, desat) * matrix.tint(tint[0], tint[1], tint[2]))
+def Grayscale(im, desat=(0.2126, 0.7152, 0.0722), **properties):
+    return MatrixColor(im, matrix.saturation(0.0, desat), **properties)
+
+
+def Sepia(im, tint=(1.0, .94, .76), desat=(0.2126, 0.7152, 0.0722), **properties):
+    return MatrixColor(im, matrix.saturation(0.0, desat) * matrix.tint(tint[0], tint[1], tint[2]), **properties)
     
+
 def Color(im, color):
     """
     This recolors the supplied image, mapping colors such that black is
@@ -1327,7 +1397,7 @@ class AlphaMask(ImageBase):
             raise Exception("AlphaMask surfaces must be the same size.")
 
         # Used to copy the surface.
-        rv = basesurf.convert_alpha()
+        rv = basesurf.convert_alpha(renpy.game.interface.display.window)
 
         if renpy.display.module.can_munge:
             renpy.display.module.alpha_munge(masksurf, rv, identity)
