@@ -127,7 +127,7 @@ class Container(renpy.display.core.Displayable):
             return
 
         self.children.pop(i)
-        self.offsets = self.list_type()
+        self.offsets = self._list_type()
 
         if self.children:
             self.child = self.children[-1]
@@ -147,7 +147,7 @@ class Container(renpy.display.core.Displayable):
     def render(self, width, height, st, at):
 
         rv = Render(width, height)
-        self.offsets = self.list_type()
+        self.offsets = self._list_type()
         
         for c in self.children:
             cr = render(c, width, height, st, at)
@@ -156,6 +156,7 @@ class Container(renpy.display.core.Displayable):
 
         return rv
 
+    
     def event(self, ev, x, y, st):
 
         children = self.children
@@ -172,6 +173,7 @@ class Container(renpy.display.core.Displayable):
                 
         return None
 
+    
     def visit(self):
         return self.children
     
@@ -425,17 +427,31 @@ class MultiBox(Container):
         
     def render(self, width, height, st, at):
 
-        t = renpy.game.interface.frame_time
+        # Do we need to adjust the child times due to our being a layer?
+        if self.layer_name or (self.layers is not None):
+            adjust_times = True
+        else:
+            adjust_times = False
 
+        xminimum = self.style.xminimum
+        if xminimum is not None:
+            width = max(width, scale(xminimum, width))
+
+        yminimum = self.style.yminimum
+        if yminimum is not None:
+            height = max(height, scale(yminimum, height))
+            
         if self.first:
-        
-            it = renpy.game.interface.interact_time
-
-            self.start_times = [ i or it for i in self.start_times ]
-            self.anim_times = [ i or it for i in self.anim_times ]
 
             self.first = False
+
+            if adjust_times:
             
+                it = renpy.game.interface.interact_time
+                
+                self.start_times = [ i or it for i in self.start_times ]
+                self.anim_times = [ i or it for i in self.anim_times ]
+                            
             layout = self.style.box_layout
 
             if layout is None:
@@ -446,19 +462,26 @@ class MultiBox(Container):
         else:
             layout = self.layout
 
-                
+
+        # Handle time adjustment, store the results in csts and cats.
+        if adjust_times:            
+            t = renpy.game.interface.frame_time
+
+            csts = [ t - start for start in self.start_times ]
+            cats = [ t - anim for anim in self.anim_times ]
+
+        else:
+            csts = [ st ] * len(self.children)
+            cats = [ at ] * len(self.children)
+            
         if layout == "fixed":
 
             self.offsets = [ ]
             
             rv = renpy.display.render.Render(width, height, layer_name=self.layer_name)
 
-
-            for child, start, anim in zip(self.children, self.start_times, self.anim_times):
-
-                cst = t - start
-                cat = t - anim
-
+            for child, cst, cat in zip(self.children, csts, cats):
+                
                 surf = render(child, width, height, cst, cat)
 
                 if surf:
@@ -466,7 +489,7 @@ class MultiBox(Container):
                     self.offsets.append(offset)
                 else:
                     self.offsets.append((0, 0))
-
+                    
             return rv
                     
         if layout == "horizontal":
@@ -491,10 +514,10 @@ class MultiBox(Container):
 
             padding = 0
 
-            for i, padding, start, anim in zip(self.children, spacings, self.start_times, self.anim_times):
-
+            for i, padding, cst, cat in zip(self.children, spacings, csts, cats):
+                
                 xoffsets.append(xo)
-                surf = render(i, remwidth, height, t - start, t - anim)
+                surf = render(i, remwidth, height, cst, cat)
 
                 sw, sh = surf.get_size()
 
@@ -551,11 +574,11 @@ class MultiBox(Container):
 
             padding = 0
 
-            for i, padding, start, anim in zip(self.children, spacings, self.start_times, self.anim_times):
+            for i, padding, cst, cat in zip(self.children, spacings, csts, cats):
 
                 yoffsets.append(yo)
 
-                surf = render(i, width, remheight, t - start, t - anim)
+                surf = render(i, width, remheight, cst, cat)
 
                 sw, sh = surf.get_size()
 
