@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -28,16 +28,12 @@
 _file = file
 
 import renpy
-
-# Many of these shouldn't be used directly.
-# from renpy.display.layout import *
-from renpy.display.text import ParameterizedText, register_sfont
+from renpy.display.text import ParameterizedText
+from renpy.display.font import register_sfont, register_mudgefont, register_bmfont
 from renpy.display.behavior import Keymap
 from renpy.display.minigame import Minigame
-# from renpy.display.image import *
 
-from renpy.curry import curry
-# from renpy.display.audio import music_start, music_stop
+from renpy.curry import curry, partial
 from renpy.audio.sound import play
 from renpy.display.video import movie_start_fullscreen, movie_start_displayable, movie_stop
 from renpy.loadsave import load, save, list_saved_games, can_load, rename_save, unlink_save, scan_saved_game
@@ -62,7 +58,7 @@ def roll_forward_info():
 def in_rollback():
     return renpy.game.log.in_rollback()
 
-def checkpoint(data=None):
+def checkpoint(data=None, keep_rollback=False):
     """
     This creates a checkpoint that the user can rollback to. The
     checkpoint is placed at the statement after the last statement
@@ -71,7 +67,8 @@ def checkpoint(data=None):
     statement.
     """
 
-    renpy.game.log.checkpoint(data)
+    if renpy.store._rollback:
+        renpy.game.log.checkpoint(data, keep_rollback=keep_rollback)
 
 def block_rollback():
     """
@@ -172,6 +169,8 @@ def showing(name, layer='master'):
 def show(name, at_list=[ ], layer='master', what=None, zorder=0, tag=None, behind=[ ]):
     "Documented in wiki as renpy.show."
 
+    if renpy.game.init_phase:
+        raise Exception("Show may not run while in init phase.")
     
     if not isinstance(name, tuple):
         name = tuple(name.split())
@@ -185,8 +184,11 @@ def show(name, at_list=[ ], layer='master', what=None, zorder=0, tag=None, behin
 
     if what is None:
         what = name
+    elif isinstance(what, basestring):
+        what = tuple(what.split())
+
+    base = img = renpy.display.image.ImageReference(what, style='image_placement')
 
-    img = renpy.display.image.ImageReference(what, style='image_placement')
     for i in at_list:
         img = i(img)
 
@@ -196,6 +198,13 @@ def show(name, at_list=[ ], layer='master', what=None, zorder=0, tag=None, behin
     if tag:
         name = (tag,) + name[1:]
     
+    if not base.find_target() and renpy.config.missing_show:
+        if renpy.config.missing_show(name, what, layer):
+            return
+
+    if renpy.config.missing_hide:
+        renpy.config.missing_hide(name, layer)
+
     sls.add(layer, img, key, zorder, behind, at_list=at_list, name=name)
     
 
@@ -214,12 +223,18 @@ def hide(name, layer='master'):
     @param layer: The layer this operates on.
     """
 
+    if renpy.game.init_phase:
+        raise Exception("Hide may not run while in init phase.")
+
     if not isinstance(name, tuple):
         name = tuple(name.split())
 
     sls = scene_lists()
     key = name[0]
     sls.remove(layer, key)
+
+    if renpy.config.missing_hide:
+        renpy.config.missing_hide(name, layer)
     
 
 def scene(layer='master'):
@@ -229,9 +244,14 @@ def scene(layer='master'):
     to then add something new, call renpy.show after this.
     """
 
+    if renpy.game.init_phase:
+        raise Exception("Scene may not run while in init phase.")
+
     sls = scene_lists()
     sls.clear(layer)
-    
+
+    if renpy.config.missing_scene:
+        renpy.config.missing_scene(layer)
         
 def watch(expression, style='default', **properties):
     """
@@ -279,10 +299,12 @@ def input(prompt, default='', allow=None, exclude='{}', length=None, with_none=N
 
     renpy.ui.close()
 
+    renpy.exports.shown_window()
+        
     roll_forward = renpy.exports.roll_forward_info()
     if not isinstance(roll_forward, basestring):
         roll_forward = None
-    
+
     rv = renpy.ui.interact(mouse='prompt', type="input", roll_forward=roll_forward)
     renpy.exports.checkpoint(rv)
     
@@ -472,7 +494,7 @@ def say(who, what, interact=True):
     or a string, it is passed directly to display_say. Otherwise, the
     say method is called on the who object with what as a parameter.
     """
-
+    
     # Interpolate variables.
     what = what % tag_quoting_dict
 
@@ -487,7 +509,7 @@ def say(who, what, interact=True):
 
 
 def imagemap(ground, selected, hotspots, unselected=None, overlays=False,
-             style='imagemap', with_none=None, **properties):
+             style='imagemap', mouse='imagemap', with_none=None, **properties):
     """
     Displays an imagemap. An image map consists of two images and a
     list of hotspots that are defined on that image. When the user
@@ -530,7 +552,7 @@ def imagemap(ground, selected, hotspots, unselected=None, overlays=False,
     
     rv = renpy.ui.interact(suppress_overlay=(not overlays),
                            type='imagemap',
-                           mouse='imagemap',
+                           mouse=mouse,
                            roll_forward=roll_forward)
 
     renpy.exports.checkpoint(rv)
@@ -568,7 +590,7 @@ def pause(delay=None, music=None, with_none=None, hard=False):
         roll_forward = None
     
     rv = renpy.ui.interact(mouse='pause', type='pause', roll_forward=roll_forward)
-    renpy.exports.checkpoint(rv)
+    renpy.exports.checkpoint(rv, keep_rollback=True)
 
 
     if with_none is None:
@@ -580,7 +602,7 @@ def pause(delay=None, music=None, with_none=None, hard=False):
     return rv
 
 
-def movie_cutscene(filename, delay=None, loops=0):
+def movie_cutscene(filename, delay=None, loops=0, stop_music=True):
     """
     This displays an MPEG-1 cutscene for the specified number of
     seconds. The user can click to interrupt the cutscene.
@@ -599,17 +621,16 @@ def movie_cutscene(filename, delay=None, loops=0):
     given delay elapsed uninterrupted.
     """
 
+    if stop_music:
+        renpy.audio.audio.set_force_stop("music", True)
+    
     movie_start_fullscreen(filename, loops=loops)
-
+    
     renpy.ui.saybehavior()
 
-    if delay is None:
-        if loops < 0:
-            delay = -1
-        else:
-            delay = renpy.display.video.movie_length(filename) * (loops + 1)
-
-    if delay >= 0:
+    if delay is None or delay < 0:
+        renpy.ui.soundstopbehavior("movie")
+    else:
         renpy.ui.pausebehavior(delay, False)
 
     if renpy.game.log.forward:
@@ -622,11 +643,14 @@ def movie_cutscene(filename, delay=None, loops=0):
                            show_mouse=False,
                            roll_forward=roll_forward)
 
-    # We don't want to put a checkpoint here, as we can't roll back from
-    # here.
-    
+    # We don't want to put a checkpoint here, as we can't roll back while
+    # playing a cutscene.
+
     movie_stop()
 
+    if stop_music:
+        renpy.audio.audio.set_force_stop("music", False)
+
     return rv
         
 
@@ -643,6 +667,9 @@ def with_statement(trans, paired=None, always=False, clear=True):
     @param always: Always perform the transition.
     """
 
+    if renpy.game.init_phase:
+        raise Exception("With statements may not run while in init phase.")
+
     if renpy.config.skipping:
         trans = None
 
@@ -658,10 +685,14 @@ def rollback():
     Rolls the state of the game back to the last checkpoint.
     """
 
+    if not renpy.store._rollback:
+        return
+    
     if not renpy.game.context().rollback:
         return
     
     if renpy.config.rollback_enabled:
+        renpy.config.skipping = None
         renpy.game.log.complete()
         renpy.game.log.rollback(1)
 
@@ -708,7 +739,7 @@ def take_screenshot(scale=None):
     
     renpy.game.interface.take_screenshot(scale)
 
-def full_restart(transition=False, label="_invoke_main_menu"):
+def full_restart(transition=False, label="_invoke_main_menu", target="_main_menu"):
     """
     This causes a full restart of Ren'Py. 
     """
@@ -716,7 +747,7 @@ def full_restart(transition=False, label="_invoke_main_menu"):
     if transition is False:
         transition = renpy.config.end_game_transition
     
-    raise renpy.game.FullRestartException((transition, label))
+    raise renpy.game.FullRestartException((transition, label, target))
 
 def utter_restart():
     """
@@ -749,7 +780,6 @@ def jump_out_of_context(label):
 
     raise renpy.game.JumpOutException(label)
 
-
 def screenshot(filename):
     """
     Saves a screenshot in the named filename.
@@ -906,13 +936,41 @@ def get_filename_line():
 
 def shell_escape(s):
     s = s.replace("\\", "\\\\")
-    # s = s.replace("\'", "\\\'")
     s = s.replace("\"", "\\\"")
-    s = s.replace("\$", "\\\$")
-    s = s.replace("\%", "\\\%")
-    s = s.replace("\`", "\\\`")
     return s
 
+def split_args(s):
+
+    rv = [ ]
+    word = ""
+    quoted = False
+        
+    i = 0
+    while i < len(s):
+        c = s[i]
+        i += 1
+
+        if not quoted and c == ' ' and word:
+            if word:
+                rv.append(word)
+                word = ""
+            continue
+            
+        if c == '"':
+            quoted = not quoted
+            continue
+
+        if c == '\\':
+            c = s[i]
+            i += 1
+
+        word += c
+
+    if word:
+        rv.append(word)
+
+    return rv
+
 def launch_editor(filenames, line=1, transient=0):
     """
     This causes an editor to be launched at the location of the current
@@ -929,21 +987,13 @@ def launch_editor(filenames, line=1, transient=0):
     if not len(filenames):
         return
 
-    # join = '" "'
     shell = True
     
-#     if hasattr(sys, "winver"):
-#         shell = False
-#         # join = '" "'
-#     else:
-#         shell = True
-#         # join = "' '"
-
     filenames = [ shell_escape(os.path.normpath(i)) for i in filenames ]
     filename = filenames[0]
 
-    allfiles =  '" "'.join(filenames)
-    otherfiles = '" "'.join(filenames[1:])
+    allfiles = config.editor_file_separator.join(filenames)
+    otherfiles = config.editor_file_separator.join(filenames[1:])
     
     subs = dict(filename=filename, line=line, allfiles=allfiles, otherfiles=otherfiles)
     if transient and (renpy.config.editor_transient is not None):
@@ -954,11 +1004,12 @@ def launch_editor(filenames, line=1, transient=0):
     cmd = cmd.replace('""', '')
 
     try:
-        return subprocess.Popen(cmd, shell=shell)
+        subprocess.Popen(split_args(cmd))
+        return True
     except:
         if renpy.config.debug:
             raise
-        return None
+        return False
 
 # A file that log logs to.
 logfile = None
@@ -1066,9 +1117,21 @@ def layer_at_list(at_list, layer='master'):
 def free_memory():
     force_full_redraw()
     renpy.display.im.free_memory()
-    renpy.display.text.free_memory()
+    renpy.display.font.free_memory()
     renpy.display.render.free_memory()
 
+def easy_displayable(d, none=False):
+    if none:
+        return renpy.easy.displayable(d)
+    else:
+        return renpy.easy.displayable_or_none(d)
+
+def quit_event():
+    renpy.game.interface.quit_event()
+
+def iconify():
+    renpy.game.interface.display.iconify()
+    
 # New context stuff.
 call_in_new_context = renpy.game.call_in_new_context
 curried_call_in_new_context = renpy.curry.curry(renpy.game.call_in_new_context)
@@ -1090,17 +1153,6 @@ def pop_error_handler():
 def error(msg):
     _error_handlers[-1](msg)
     
-# User-defined Displayable stuff.
-
-# Displayable = renpy.display.core.Displayable
-
-class Displayable(renpy.display.core.Displayable, renpy.python.RevertableObject):
-    pass
-
-Render = renpy.display.render.Render
-render = renpy.display.render.render
-IgnoreEvent = renpy.display.core.IgnoreEvent
-redraw = renpy.display.render.redraw
 
 def timeout(seconds):
     renpy.game.interface.timeout(seconds)
@@ -1113,6 +1165,8 @@ def scry():
 def munged_filename():
     return renpy.parser.munge_filename(get_filename_line()[0])
 
+# Module loading stuff.
+
 loaded_modules = set()
 
 def load_module(name, **kwargs):
@@ -1149,3 +1203,40 @@ def pop_return():
     renpy.game.context().pop_dynamic()
     renpy.game.context().lookup_return(pop=True)
     
+def game_menu(screen=None):
+    if screen is None:
+        call_in_new_context("_game_menu")
+    else:
+        call_in_new_context("_game_menu", screen)
+
+def shown_window():
+    renpy.game.interface.shown_window = True
+
+def get_placement(d):
+    o = renpy.store.object()
+    p = d.get_placement()
+
+    o.xpos = d[0]
+    o.ypos = d[1]
+    o.xanchor = d[2]
+    o.yanchor = d[3]
+    o.xoffset = d[4]
+    o.yoffset = d[5]
+    o.subpixel = d[6]
+
+    return o
+    
+# User-Defined Displayable stuff.
+
+Render = renpy.display.render.Render
+render = renpy.display.render.render
+IgnoreEvent = renpy.display.core.IgnoreEvent
+redraw = renpy.display.render.redraw
+
+class Displayable(renpy.display.core.Displayable, renpy.python.RevertableObject):
+    pass
+
+class Container(renpy.display.core.Displayable, renpy.python.RevertableObject):
+    _list_type = renpy.python.RevertableList
+
+    
