@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -33,11 +33,13 @@ import marshal
 import random
 import types
 import weakref
+import re
 import sets
 import sys
 
 import renpy
 
+
 ##### Code that computes reachable objects, which is used to filter
 ##### the rollback list before rollback or serialization.
 
@@ -103,6 +105,7 @@ def reached_vars(store, reachable, wait):
     
     for c in renpy.game.contexts:
         reached(c.info, reachable, wait)
+        reached(c.music, reachable, wait)
         for d in c.dynamic_stack:
             for v in d.itervalues():
                 reached(v, reachable, wait)
@@ -167,6 +170,31 @@ def set_filename(filename, offset, tree):
         worklist.extend(node.getChildNodes())
 
 
+def make_unicode(m):
+    """
+    If the string s contains a unicode character, make it into a
+    unicode string.
+    """
+
+    s = m.group(0)
+
+    if "\\u" not in s:
+        return s
+    
+    prefix = m.group(1)
+    sep = m.group(2)
+    body = m.group(3)
+
+    if "u" not in prefix and "U" not in prefix:
+        prefix = 'u' + prefix
+
+    rv = prefix + sep + body + sep
+    return rv
+    
+
+string_re = re.compile(r'([uU]?[rR]?)("""|"|\'\'\'|\')((\\.|.)*?)\2')
+
+
 def py_compile(source, mode, filename='<none>', lineno=1):
     """
     Compiles the given source code using the supplied codegenerator.
@@ -185,10 +213,14 @@ def py_compile(source, mode, filename='<none>', lineno=1):
     if isinstance(source, renpy.ast.PyExpr):
         filename = source.filename
         lineno = source.linenumber
-    
+
+    orig_source = source
     source = source.replace("\r", "")
     source = source.encode('raw_unicode_escape')
 
+    if "\\u" in source:
+        source = string_re.sub(make_unicode, source)
+    
     try:
         tree = parse(source, mode)
     except SyntaxError, e:
@@ -197,8 +229,8 @@ def py_compile(source, mode, filename='<none>', lineno=1):
             msg = "Syntax error on line %d of %s" % (e.lineno + lineno - 1, filename)
 
             if len(source) < 128:
-                msg += ":\n    " + source
-            
+                msg += ":\n    " + orig_source
+
             raise Exception(msg)
         else:
             raise
@@ -273,7 +305,6 @@ class RevertableList(list):
     __getslice__ = wrapper(list.__getslice__)
     __mul__ = wrapper(list.__mul__)
 
-    # TODO: Handle __iter__ (or not).
     def get_rollback(self):
         return self[:]
 
@@ -504,6 +535,7 @@ class Rollback(renpy.object.Object):
         for o, rb in self.objects:
             if id(o) in reachable:
                 new_objects.append((o, rb))
+                reached(rb, reachable, wait)
             else:
                 if renpy.config.debug:
                     print "Removing unreachable:", o
@@ -569,13 +601,19 @@ class RollbackLog(renpy.object.Object):
         self.ever_been_changed = { }
         self.rollback_limit = 0
         self.forward = [ ]
+
+        # Did we just do a roll forward?
+        self.rolled_forward = False
+
         
         # Reset the RNG on the creation of a new game.
         rng.reset()
 
     def after_setstate(self):
         self.mutated = { }
-
+        self.rolled_forward = False
+        
+        
     def begin(self):
         """
         Called before a node begins executing, to indicate that the
@@ -607,6 +645,8 @@ class RollbackLog(renpy.object.Object):
         # save code.
         global mutate_flag
         mutate_flag = True
+
+        self.rolled_forward = False
         
     def complete(self):
         """
@@ -706,7 +746,7 @@ class RollbackLog(renpy.object.Object):
         return None
             
 
-    def checkpoint(self, data=None):
+    def checkpoint(self, data=None, keep_rollback=False):
         """
         Called to indicate that this is a checkpoint, which means
         that the user may want to rollback to just before this
@@ -728,7 +768,10 @@ class RollbackLog(renpy.object.Object):
                 # Otherwise, clear the forward stack.
                 fwd_name, fwd_data = self.forward[0]
 
-                if (self.current.context.current == fwd_name and data == fwd_data):
+                if (self.current.context.current == fwd_name
+                    and data == fwd_data
+                    and (keep_rollback or self.rolled_forward)
+                    ):
                     self.forward.pop(0)
                 else:
                     self.forward = [ ]
@@ -736,8 +779,6 @@ class RollbackLog(renpy.object.Object):
             # Log the data in case we roll back again.
             self.current.forward = data
 
-            
-        
     def block(self):
         """
         Called to indicate that the user should not be able to rollback
