@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -443,7 +443,7 @@ class Script(object):
         if self.bytecode_dirty:
             try:
                 data = (BYTECODE_VERSION, self.bytecode_newcache)
-                f = file(os.path.join(renpy.config.searchpath[0], "bytecode.rpyb"), "w")
+                f = file(os.path.join(renpy.config.searchpath[0], "bytecode.rpyb"), "wb")
                 f.write(dumps(data, 2).encode("zlib"))
                 f.close()
             except:
